@@ -377,33 +377,61 @@ func (c *gitlabClient) Upload(
 		projectID = ctx.Config.Release.GitLab.Owner + "/" + projectID
 	}
 
-	log.WithField("file", file.Name()).Debug("uploading file")
-	projectFile, _, err := c.client.Projects.UploadFile(
-		projectID,
-		file.Name(),
-		nil,
-	)
-	if err != nil {
-		return err
+	var baseLinkURL string
+	var linkURL string
+	if ctx.Config.GitLabURLs.UsePackageRegistry {
+		log.WithField("file", file.Name()).Debug("uploading file as generic package")
+		if _, _, err := c.client.GenericPackages.PublishPackageFile(
+			projectID,
+			ctx.Config.ProjectName,
+			ctx.Version,
+			artifact.Name,
+			file,
+			nil,
+		); err != nil {
+			return err
+		}
+
+		baseLinkURL, err = c.client.GenericPackages.FormatPackageURL(
+			projectID,
+			ctx.Config.ProjectName,
+			ctx.Version,
+			artifact.Name,
+		)
+		if err != nil {
+			return err
+		}
+		linkURL = c.client.BaseURL().String() + baseLinkURL
+	} else {
+		log.WithField("file", file.Name()).Debug("uploading file as attachment")
+		projectFile, _, err := c.client.Projects.UploadFile(
+			projectID,
+			file.Name(),
+			nil,
+		)
+		if err != nil {
+			return err
+		}
+
+		baseLinkURL = projectFile.URL
+		gitlabBaseURL, err := tmpl.New(ctx).Apply(ctx.Config.GitLabURLs.Download)
+		if err != nil {
+			return fmt.Errorf("templating GitLab Download URL: %w", err)
+		}
+
+		// search for project details based on projectID
+		projectDetails, _, err := c.client.Projects.GetProject(projectID, nil)
+		if err != nil {
+			return err
+		}
+		linkURL = gitlabBaseURL + "/" + projectDetails.PathWithNamespace + baseLinkURL
 	}
 
 	log.WithFields(log.Fields{
 		"file": file.Name(),
-		"url":  projectFile.URL,
+		"url":  baseLinkURL,
 	}).Debug("uploaded file")
 
-	// search for project details based on projectID
-	projectDetails, _, err := c.client.Projects.GetProject(projectID, nil)
-	if err != nil {
-		return err
-	}
-
-	gitlabBaseURL, err := tmpl.New(ctx).Apply(ctx.Config.GitLabURLs.Download)
-	if err != nil {
-		return fmt.Errorf("templating GitLab Download URL: %w", err)
-	}
-
-	linkURL := gitlabBaseURL + "/" + projectDetails.PathWithNamespace + projectFile.URL
 	name := artifact.Name
 	filename := "/" + name
 	releaseLink, _, err := c.client.ReleaseLinks.CreateReleaseLink(
